'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = configure;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _vm = require('vm');

var _vm2 = _interopRequireDefault(_vm);

var _resolve = require('resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _babelCore = require('babel-core');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var called = false;
function configure(entrypoint, overrideRequires, opts) {
  if (called) throw new Error('You can only use babel-live once per project');
  called = true;

  opts = opts || {};
  if (opts.sourceMap !== false) opts.sourceMap = "inline";

  var requireCache = {};
  // filename => fn(module, exports, require, __filename, __dirname)
  var moduleCache = {};
  function invalidate(filename) {
    requireCache = {};
    moduleCache[filename] = null;
    babelRequire(entrypoint);
  }

  var watching = {};
  function watch(filename) {
    if (watching[filename]) return;
    watching[filename] = true;
    var w = _chokidar2.default.watch(filename, {
      persistent: true,
      usePolling: true,
      interval: 100
    });
    w.on('error', function (err) {
      console.log('error watching file');
      console.log('probably nothing to worry about');
      console.log(err.message);
    });
    w.on('change', function () {
      return invalidate(filename);
    });
  }

  function babelRequire(filename) {
    filename = _path2.default.resolve(filename);
    watch(filename);
    if (requireCache[filename]) return requireCache[filename];
    var fn = moduleCache[filename];
    if (!fn) {
      var src = babelLoad(filename);
      fn = _vm2.default.runInThisContext('(function(module,exports,require,__filename,__dirname){' + src + '\n})', filename);
      moduleCache[filename] = fn;
    }
    requireCache[filename] = {};
    var mod = {
      exports: requireCache[filename]
    };
    function proxiedRequire(id) {
      if (overrideRequires[id]) {
        return overrideRequires[id];
      }
      var p = _resolve2.default.sync(id, {
        basedir: _path2.default.dirname(filename),
        extensions: ['.js', '.json']
      });
      if (/^\./.test(id)) {
        return babelRequire(p);
      } else {
        return require(p);
      }
    }
    proxiedRequire.resolve = function (id) {
      return _resolve2.default.sync(id, {
        basedir: _path2.default.dirname(filename),
        extensions: ['.js', '.json']
      });
    };
    var sandbox = {
      'module': mod,
      'exports': mod.exports,
      'require': proxiedRequire,
      '__filename': filename,
      '__dirname': _path2.default.dirname(filename)
    };
    fn(sandbox.module, sandbox.exports, sandbox.require, sandbox.__filename, sandbox.__dirname);
    return requireCache[filename] = mod.exports;
  }

  function babelLoad(filename) {
    var src = _fs2.default.readFileSync(filename, 'utf8');
    opts.filename = filename;
    return (0, _babelCore.transform)(src, opts).code;
  }

  return babelRequire(entrypoint);
}